package com.bcxin.event.core.definitions;

import com.bcxin.event.core.metas.TableMeta;
import lombok.AccessLevel;
import lombok.Data;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.swing.table.TableModel;
import java.util.Collection;
import java.util.Collections;
import java.util.stream.Collectors;

@Data
public class JdbcTableDefinition extends TableDefinitionAbstract {
    private static final Logger logger = LoggerFactory.getLogger(JdbcTableDefinition.class);

    private JdbcConnectorDefinition connector;

    private String definitionSql;

    /**
     * 系统自动生成的定义脚本
     * definition sql
     */
    @Getter(AccessLevel.PRIVATE)
    @Setter(AccessLevel.PRIVATE)
    private Collection<String> autoGeneratedDefinitionSql;

    @Override
    public String getSql() {
        if (this.getConnector() == null) {
            throw new IllegalArgumentException("JdbcTableDefinition不能为空");
        }

        if (this.getDefinitionSql() == null) {
            throw new IllegalArgumentException(String.format("JdbcTableDefinition(%s)的definitionSql不能为空",
                    this.getConnector().getUrl()));
        }

        return String.format("%s WITH (%s)", this.getDefinitionSql(),
                this.getConnector().getJdbcDefinitionMeta());
    }

    public void build(TableMeta tableMeta) {
        if (this.getConnector() == null) {
            throw new IllegalArgumentException("JdbcTableDefinition不能为空");
        }

        if (StringUtils.isEmpty(this.getDefinitionSql())) {
            logger.warn("JdbcTableDefinition({})的definitionSql不能为空", this.getConnector().getUrl());
            return;
        }
    }
}
