package com.bcxin.event.core.definitions;

import com.bcxin.event.core.exceptions.IllegalArgumentEventException;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

@Data
public class KafkaTopicConnectorDefinition extends ConnectorDefinitionAbstract {
    private static final String KAFKA_SOURCE_PRIMARY_KEY_NAME="id";

    private String name;

    private String connector;
    private String bootstrapServer;
    private String topic;
    private String consumerGroupId;
    private String autoOffsetRest = "earliest";
    private String scanStartupMode = "group-offsets";
    private String enableAutoCommit = "true";
    private String primaryKeyName=KAFKA_SOURCE_PRIMARY_KEY_NAME;

    private String format;

    @Override
    public String getConnector() {
        return ConnectorConstants.KAFKA;
    }

    public String getDefinitionMeta() {
        return String.format(
                "'connector' = '%s'," +
                        "'properties.group.id'='%s'," +
                        "'topic' = '%s'," +
                        "'properties.bootstrap.servers' = '%s'," +
                        //https://blog.csdn.net/ASN_forever/article/details/106405041
                        // "'properties.auto.offset.reset' = 'earliest'," +
                        //"'properties.enable.auto.commit' = 'true'," +
                        "'scan.startup.mode' = 'latest-offset'," +
                        "'format'='%s'",
                StringUtils.isEmpty(this.getConnector()) ? "kafka" : this.getConnector(),
                this.getConsumerGroupId(),
                this.getTopic(),
                this.getBootstrapServer(),
                StringUtils.isEmpty(this.getFormat()) ? "debezium-json" : this.getFormat()
        );
    }

    public String getPrimaryKeyName() {
        if (StringUtils.isEmpty(this.primaryKeyName)) {
            return KAFKA_SOURCE_PRIMARY_KEY_NAME;
        }

        return this.primaryKeyName;
    }
}
