package com.bcxin.event.core.utils;

import com.alibaba.fastjson.JSONObject;
import com.bcxin.event.core.FlinkConstants;
import org.apache.commons.lang3.StringUtils;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.util.*;
import java.util.stream.Collectors;

public class DebeziumUtil {
    public static final String META_TABLE_NAME = "table";
    public static final String META_DB_NAME = "db";

    public static final String SYNC_DATE_TIME = "cdc_sync_time";
    public static final String META_ROW_DOMAIN_ID_NAME = "domainid";
    public static final String META_ROW_ID_NAME = "id";
    public static final String META_ROW_LASTMODIFIED_NAME = "lastmodified";

    public static Map<String, String> getCleanFlatMapResult(JSONObject data) {
        Map<String, String> flatMap = new HashMap<>();

        JSONObject sourceMap = data.getJSONObject("source");
        for (String key : sourceMap.keySet()) {
            if (sourceMap.get(key) != null) {


                putValue(flatMap, String.format("source.%s", key), String.valueOf(sourceMap.get(key)));
            }
        }

        String tableName = (String) sourceMap.get(META_TABLE_NAME);
        String dbName = (String) sourceMap.get(META_DB_NAME);
        Object domainId = null;
        Object id = null;
        Object LASTMODIFIED = null;

        if (data.getJSONObject("before") != null) {
            JSONObject before = data.getJSONObject("before");
            domainId = getValue(before, META_ROW_DOMAIN_ID_NAME);
            id = getValue(before, META_ROW_ID_NAME);
            LASTMODIFIED = getValue(before, META_ROW_LASTMODIFIED_NAME);
            for (String key : before.keySet()) {
                if (before.get(key) != null) {
                    putValue(flatMap, String.format("before.%s", key), String.valueOf(before.get(key)));
                }
            }

            if (domainId == null) {
                domainId = getValue(before, "organization_id");
            }
        }

        if (data.getJSONObject("after") != null) {
            JSONObject after = data.getJSONObject("after");
            domainId = getValue(after, "domainId");
            id = getValue(after, "id");
            LASTMODIFIED = getValue(after, "LASTMODIFIED");


            for (String key : after.keySet()) {
                if (after.get(key) != null) {
                    putValue(flatMap, String.format("after.%s", key), String.valueOf(after.get(key)));
                }
            }

            if (domainId == null) {
                domainId = getValue(after, "organization_id");
            }
        }

        if (domainId != null) {
            putValue(flatMap, META_ROW_DOMAIN_ID_NAME, String.valueOf(domainId));
        }
        if (id != null) {
            putValue(flatMap, META_ROW_ID_NAME, String.valueOf(id));
        }

        if (!StringUtils.isEmpty(tableName)) {
            putValue(flatMap, META_TABLE_NAME, tableName);
        }
        if (!StringUtils.isEmpty(dbName)) {
            putValue(flatMap, META_DB_NAME, dbName);
        }

        if (LASTMODIFIED != null) {
            putValue(flatMap, META_ROW_LASTMODIFIED_NAME, String.valueOf(LASTMODIFIED));
        }

        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
        putValue(flatMap, SYNC_DATE_TIME, dateFormat.format(Timestamp.from(Instant.now())));

        return flatMap;
    }

    private static Object getValue(JSONObject data, String fieldName) {
        if (data == null || StringUtils.isEmpty(fieldName)) {
            return null;
        }

        Optional<String> selectedFieldNameOptional =
                data.keySet().stream().filter(ix -> ix.equalsIgnoreCase(fieldName)).findFirst();
        if (!selectedFieldNameOptional.isPresent()) {
            return null;
        }

        return data.get(selectedFieldNameOptional.get());
    }

    private static void putValue(Map<String, String> flatMap, String key, String value) {
        if (StringUtils.isEmpty(key) || value == null) {
            return;
        }

        if (FlinkConstants.isDateTimeField(key)) {
            flatMap.put(key.toLowerCase(), FlinkConstants.formatValue(key, value));
        }

        flatMap.put(key.toLowerCase(), value);
    }

    public static String getMapValue(Map<String, String> data, String key) {
        if (StringUtils.isEmpty(key) || data == null) {
            return null;
        }

        Optional<String> selectedFieldNameOptional =
                data.keySet().stream().filter(ix -> ix.equalsIgnoreCase(key)).findFirst();
        if (!selectedFieldNameOptional.isPresent()) {
            return null;
        }

        return data.get(selectedFieldNameOptional.get());
    }

    public static Date getDateFromDateType(String dayValue) {
        if (com.bcxin.event.core.utils.StringUtils.hasAlpha(dayValue)) {
            return null;
        }

        try {
            long valueInMils = 0;
            if (dayValue.length() < 8) {
                valueInMils = translate2ValueInMils(Long.valueOf(dayValue));
            } else {
                valueInMils = Long.valueOf(dayValue);
            }

            return new Date(valueInMils);
        } catch (Exception ex) {
            return null;
        }
    }

    public static long translate2ValueInMils(long valueInDay) {
        return valueInDay * 24 * 60 * 60 * 1000;
    }
}
